/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */

//#ifdef __WITH_DATABINDING

/**
 * @attribute {String}  icon
 * @attribute {String}  caption
 * @attribute {String}  width
 * @attribute {String}  options
 * @attribute {String}  editor
 * @attribute {String}  colspan
 * @attribute {String}  align
 * @attribute {String}  css
 * @attribute {Boolean} tree
 */
ppc.BindingColumnRule = function(struct, tagName){
    this.$init(tagName || "column", ppc.NODE_VISIBLE, struct);
    
    this.$className = "col" + this.$uniqueId;
};

(function(){
    this.$defaultwidth = "100";
    this.$width        = 0;
    
    this.$sortable  = true; //@todo set defaults based on localName of element to which its applied
    this.$resizable = true;
    this.$movable   = true;
    this.$cssInit   = false;
    
    this.visible    = true;
    
    //1 = force no bind rule, 2 = force bind rule
    this.$attrExcludePropBind = ppc.extend({
        css         : 1,
        icon        : 1,
        caption     : 1,
        eachcaption : 1,
        eachvalue   : 1,
        each        : 1,
        icon        : 1
    }, this.$attrExcludePropBind);
    
    this.$supportedProperties.push("tree", "icon", "caption", "width", "options", 
        "check", "editor", "colspan", "align", "css", "sorted", "each", 
        "eachvalue", "eachcaption", "model");
    
    this.$booleanProperties["tree"]    = true;
    this.$booleanProperties["check"]   = true;
    this.$booleanProperties["sorted"]  = true;
    this.$booleanProperties["visible"] = true;
    
    this.$setParentFixedWidth = function(){
        var pNode = this.parentNode;
        var vLeft = (pNode.$fixed) + "px";
        if (!this.$isFixedGrid) {
            //ppc.setStyleRule("." + this.$baseCSSname + " .headings ." + hFirst.$className, "marginLeft", "-" + vLeft); //Set
            //ppc.setStyleRule("." + this.$baseCSSname + " .records ." + hFirst.$className, "marginLeft", "-" + vLeft); //Set
            ppc.setStyleRule("." + pNode.$baseCSSname + " .row" + pNode.$uniqueId,
                "paddingRight", vLeft, null, this.oWin); //Set
            ppc.setStyleRule("." + pNode.$baseCSSname + " .row" + pNode.$uniqueId,
                "marginRight", "-" + vLeft, null, pNode.oWin); //Set
        
            //headings and records have same padding-right
            if (pNode.$container)
                pNode.$container.style.paddingRight = vLeft;
            if (pNode.$head)
                pNode.$head.style.paddingRight = vLeft;
        }
    }
    
    this.$propHandlers["width"]  = function(value, prop){
        if (!value)
            return;

        var diff = value - this.$width;

        this.$isPercentage = value && String(value).indexOf("%") > -1;
        this.$width = parseFloat(value);

        var pNode = this.parentNode;
        
        if (this.$isPercentage) {
            ppc.setStyleRule("." + this.$className, "width", this.$width + "%");
            
            //if (ppc.z && !this.resizing)
                //this.resize(this.$width, pNode, true);
        }
        else {
            ppc.setStyleRule("." + this.$className, "width", this.$width + "px", null, pNode.oWin); //Set
            
            if (pNode.$amlLoaded) {
                if (this.visible)
                    pNode.$fixed += diff;
                
                this.$setParentFixedWidth();
            }
        }
    }
    
    this.$propHandlers["options"]  = function(value, prop){
        this.$sortable  = value.indexOf("sort") > -1;
        this.$resizable = value.indexOf("size") > -1;
        this.$movable   = value.indexOf("move") > -1;
    }
    
    this.$propHandlers["visible"] = function(value, prop, el, force){
        var pNode = this.parentNode;
        
        if (!force && !this.$amlLoaded)
            return;

        if (value) {
            ppc.setStyleRule("." + this.$className,
                "display", "inline-block", null, this.oWin);
            
            var size = this.$isPercentage
                ? (this.$ext.offsetWidth - (pNode.$widthdiff - 3))
                : this.$width;

            this.resize(size, pNode, true);
        }
        else {
            ppc.setStyleRule("." + this.$className,
                "display", "none", null, this.oWin);

            this.resize(0, pNode, true);
        }
    }
    
    this.resize = function(newsize, pNode, toggleShowHide){
        var hN;
        
        if (this.$isPercentage) {
            var oldsize = (this.visible && this.$ext.offsetWidth
              ? this.$ext.offsetWidth - (pNode.$widthdiff - 3)
              : 0),
                ratio = oldsize ? newsize / oldsize : 1, //div 0 ??
                next  = [],
                fixed = [],
                total = 0,
                node  = toggleShowHide 
                    ? this.$ext.parentNode.firstChild 
                    : this.$ext.nextSibling;
            
            while (node && node.getAttribute("hid")) {
                hN = ppc.all[node.getAttribute("hid")];
                if (hN.visible !== false) {
                    if (hN.$isPercentage) {
                        next.push(hN);
                        total += hN.$width;
                    }
                    else fixed.push(hN);
                }
                node = node.nextSibling;
            }
            
            if (fixed.length && !next.length)
                return fixed[0].resize(fixed[0].$width + (oldsize - newsize), pNode);

            var diffPerc, diffRatio;
            if (ratio == 1 && total < 101) {
                ratio = total/101;
                diffRatio = 1/ratio;
            }
            else if (total > 101) {
                ratio = ratio * 101/total
                diffRatio = ratio;
            }
            else {
                diffPerc = (ratio - 1) * this.$width;
                diffRatio = (total - diffPerc) / total;
                if (diffRatio < 0.01 && diffRatio > 0) {
                    if (newsize < 20) return;
                    return this.resize(newsize - 10, pNode);//pNode.resizeColumn(nr, newsize - 10);
                }
            }
            
            for (var n, i = 0; i < next.length; i++) {
                n = next[i];
                if (n == this)
                    continue;
                
                n.setProperty("width", String(n.$width * diffRatio) + "%", false, true);
            }
            
            if (this.visible !== false) {
                this.setProperty("width", String(ratio * this.$width) + "%", false, true);
            }
        }
        else if (toggleShowHide) {
            var diff = newsize;
            pNode.$fixed += diff;
            this.$setParentFixedWidth();
        }
        else {
            if (ppc.isIE && pNode.oIframe)
                this.$ext.style.width = newsize + "px";

            this.setProperty("width", newsize, false, true);
        }
    };
    
    this.hide = function(){
        this.setProperty("visible", false, false, true);
        return this;
    };
    
    this.show = function(){
        this.setProperty("visible", true, false, true);
        return this;
    };
    
    /**
     * Sorts a column.
     * @param {Number} hid the heading number; this number is based on the sequence of the column elements.
     */
    this.sort = function(pNode, initial){
        if (pNode.$lastSorted == this) {
            ppc.setStyleClass(this.$int,
                pNode.toggleSortOrder()
                    ? "ascending"
                    : "descending", ["descending", "ascending"]);
            return;
        }

        var h;
        if (h = pNode.$lastSorted) {
            ppc.setStyleRule("." + h.$className, "background", "white"); //This breaks row coloring
            ppc.setStyleClass(h.$int, "", ["descending", "ascending"]);
        }
        
        ppc.setStyleRule("." + this.$className, "background", "#f3f3f3");
        ppc.setStyleClass(this.$int, "ascending", ["descending", "ascending"]);

        pNode.resort({
            order : "ascending",
            xpath : (this.cvalue || this.compile("value")).xpaths[1],
            type : this["data-type"]
        }, false, initial || !pNode.length);
        
        
        //@todo needs more thought
        /*if (pNode.$lastSorted)
            pNode.$lastSorted.setProperty("sorted", false);
        this.setProperty("sorted", true);*/
        
        pNode.$lastSorted = this;
    };
    
    /**
     * Moves a column to another position.
     * @param {Number} fromHid the heading number of the column to move; this number is based on the sequence of the column elements.
     * @param {Number} toHid   the position the column is moved to;
     */
    this.move = function(hTo, pNode){
        if (hTo && this == hTo) 
            return;
        
        var hFrom       = this,
            childNrFrom = ppc.getChildNumber(hFrom.$int),
            childNrTo   = hTo && ppc.getChildNumber(hTo.$int);

        pNode.$head.insertBefore(hFrom.$int, hTo && hTo.$int || null);

        if (!pNode.length)
            return;

        (function _recur(nodes){
            for (var node, i = 0; i < nodes.length; i++) {
                if (nodes[i].nodeType != 1)
                    continue;
                //if (pNode.$withContainer && ((i+1) % 2) == 0)
                    //continue;
    
                node = nodes[i];
                if (pNode.$isTreeArch && node.tagName == "BLOCKQUOTE") { //@todo small hack
                    _recur(node.childNodes);
                }
                else {
                    node.insertBefore(node.childNodes[childNrFrom], 
                        childNrTo != undefined && node.childNodes[childNrTo] || null);
                }
            }
        })(pNode.$container.childNodes);
        
        /*if (this.$first == from || this.$first == to) {
            var hReset = this.$first == from ? hFrom : hTo;
            
            ppc.setStyleRule("." + this.$baseCSSname + " .headings ."
                + hReset.className, "marginLeft", "-5px"); //Reset
            ppc.setStyleRule("." + this.$baseCSSname + " .records ."
                + hReset.className, "marginLeft", "-5px"); //Reset
            
            this.$first = pNode.$head.firstChild.getAttribute("hid");
            var h = headings[this.$first];
            var vLeft = "-" + (this.$fixed + 5) + "px";

            ppc.setStyleRule("." + this.$baseCSSname + " .headings ."
                + h.className, "marginLeft", vLeft); //Set
            ppc.setStyleRule("." + this.$baseCSSname + " .records ."
                + h.className, "marginLeft", vLeft); //Set
        }*/
    }
    
    this.$draw = function(pNode, caption, width, className){
        //Find the parent this rule works on
        var pNode = pNode || this.parentNode;
        while (pNode.$bindingRule)
            pNode = pNode.parentNode;
        
        if (!pNode.hasFeature(ppc.__PRESENTATION__))
            return;
            
        if (width) 
            this.$propHandlers["width"].call(this, width);

        //"." + this.$baseCSSname + " .headings 
        //if initial
        //only needs once if this works

        ppc.importStylesheet([
            ["." + this.$className,
                "width:" + this.$width + (this.$isPercentage ? "%;" : "px;")
                + "text-align:" + this.align + ";display: inline-block"],
        ]);
        
        //Add to htmlRoot
        pNode.$getNewContext("headitem");
        var $head = pNode.$getLayoutNode("headitem");
        $head.setAttribute("class", this.$className + (className ? " " + className : ""));
        $head.setAttribute("hid", this.$uniqueId);
        
        var hCaption = pNode.$getLayoutNode("headitem", "caption");
        /*if (this.icon) {
            this.$sortable = false;
            $head.setAttribute("style", "background-image:url("
                + ppc.getAbsolutePath(pNode.iconPath, this.icon) 
                + ")");
            hCaption.nodeValue = "&nbsp;";
        }
        else*/
            hCaption.nodeValue = this.caption || caption || '&nbsp';
        
        this.$ext = this.$int = ppc.insertHtmlNode($head, pNode.$head || pNode.$container);
        
        var dragging = false;
        var _self    = this;
        
        if (this.sorted)
            this.sort(pNode, true);
        
        /*this.$int.onmouseover = function(e){
            if (!e) e = event;
            
            if (pNode.disabled) return;
            
            clearTimeout(this.$timer);

            ppc.setStyleClass(this, "hover", ["down"]);
        };*/
        
        this.$int.onmouseup = function(e){
            if (!e) e = event;
            
            if (pNode.disabled || !ppc.isChildOf(dragging, this, true)) 
                return;
            
            ppc.setStyleClass(this, "hover", ["down"]);

            if (_self.$sortable)
                _self.sort(pNode);
            
            //@todo pNode or Self?
            pNode.dispatchEvent("sortcolumn", _self);
        };
        
        this.$int.onmousedown = function(e){
            if (!e) e = event;
            dragging = target = this;
            
            if (pNode.disabled) return;

            //Resizing
            var pos   = ppc.getAbsolutePosition(target),
                sLeft = pNode.$head.scrollLeft;
            var d     = e.clientX - pos[0] + sLeft;
            if (d < 4 || target.offsetWidth - d - 8 < 3
              && ppc.getChildNumber(_self.$int) < pNode.$headings.length - 1) {
                var t = d < 4 && target.previousSibling || target;
                
                if (_self.$resizable) {
                    pos   = ppc.getAbsolutePosition(t);
                    ppc.setStyleClass(pNode.$pointer, "size_pointer", ["move_pointer"]);
                    pNode.$pointer.style.display = "block";
                    pNode.$pointer.style.left    = (t.offsetLeft - sLeft - 1) + "px";
                    pNode.$pointer.style.width   = (t.offsetWidth - pNode.$widthdiff + 1) + "px";
                    
                    // #ifdef __WITH_PLANE
                    ppc.plane.show(pNode.$pointer, null, true);
                    // #endif

                    dragging = true;
                    document.onmouseup = function(){
                        if (!e) e = event;
    
                        document.onmouseup = 
                        document.onmousemove = null;
                        
                        ppc.all[t.getAttribute("hid")].resize(pNode.$pointer.offsetWidth, pNode);
                        
                        dragging = false;
                        pNode.$pointer.style.display = "none";
                        
                        // #ifdef __WITH_PLANE
                        ppc.plane.hide();
                        // #endif

                    };
                    
                    document.onmousemove = function(e){
                        if (!e) e = event;

                        pNode.$pointer.style.width = Math.max(10, 
                            Math.min(pNode.$container.offsetWidth - pNode.$pointer.offsetLeft - 20, 
                                e.clientX - pos[0] - 1 + sLeft)) + "px";
                    };
                    
                    return;
                }
            }
            
            ppc.setStyleClass(target, "down", ["hover"]);
            
            //Moving
            if (!_self.$movable) {
                document.onmouseup = function(e){
                    document.onmouseup = null;
                    dragging = false;
                };
                
                return;
            }
            
            ppc.setStyleClass(pNode.$pointer, "move_pointer", ["size_pointer"]);
            
            var x = e.clientX - target.offsetLeft, sX = e.clientX,
                y = e.clientY - target.offsetTop,  sY = e.clientY,
                copy;
            
            document.onmouseup = function(e){
                if (!e) e = event;
                
                document.onmouseup   =
                document.onmousemove = null;
                
                dragging = false;
                pNode.$pointer.style.display = "none";
                
                if (!copy)
                    return;
                    
                copy.style.top = "-100px";
                
                var el = document.elementFromPoint(e.clientX, e.clientY);
                if (el.parentNode == copy.parentNode) {
                    var pos = ppc.getAbsolutePosition(el);
                    var beforeNode = (e.clientX - pos[0] > el.offsetWidth / 2
                        ? el.nextSibling
                        : el);

                    _self.move(beforeNode ? ppc.all[beforeNode.getAttribute("hid")] : null, pNode);
                }
                
                ppc.destroyHtmlNode(copy);
            };

            document.onmousemove = function(e){
                if (!e) e = event;
                
                if (!copy) {
                    if (Math.abs(e.clientX - sX) < 3 && Math.abs(e.clientY - sY) < 3)
                        return;
                    
                    copy = target.cloneNode(true);
                    copy.style.position = "absolute";
                    var diff = ppc.getWidthDiff(target);
                    copy.style.width    = (target.offsetWidth - diff
                        - pNode.$widthdiff + 2) + "px";
                    copy.style.left     = target.offsetLeft;
                    copy.style.top      = target.offsetTop;
                    copy.style.margin   = 0;
                    copy.removeAttribute("hid")
                    
                    ppc.setStyleClass(copy, "drag", ["ascending", "descending"]);
                    target.parentNode.appendChild(copy);
                }
                
                copy.style.top               = "-100px";
                pNode.$pointer.style.display = "none";
                
                var el = document.elementFromPoint(e.clientX, e.clientY);
                if (el.parentNode == copy.parentNode) {
                    var pos = ppc.getAbsolutePosition(el);
                    pNode.$pointer.style.left = (el.offsetLeft 
                        + ((e.clientX - pos[0] > el.offsetWidth / 2)
                            ? el.offsetWidth - 8
                            : 0)) + "px";
                    pNode.$pointer.style.display = "block";
                }
                
                copy.style.left = (e.clientX - x) + 'px';
                copy.style.top  = (e.clientY - y) + 'px';
            };
        };
        
        this.$int.onmouseout = function(e){
            if (!e) e = event;

            if (pNode.disabled) 
                return;
            
            var _self = this;
            this.$timer = setTimeout(function(){
                pNode.$ext.style.cursor = "";
                ppc.setStyleClass(_self, "", ["hover", "down"]);
            }, 10);
        };
        
        this.$int.onmousemove = function(e){
            if (dragging || pNode.disabled)
                return;
                
            if (!e) e = event;

            var pos   = ppc.getAbsolutePosition(this),
                sLeft = pNode.$head.scrollLeft;
            var d = e.clientX - pos[0] + sLeft;

            if (d < 4 || this.offsetWidth - d - pNode.$widthdiff < 3 
              && ppc.getChildNumber(_self.$int) < pNode.$headings.length - 1) {
                var t = d < 4 ? this.previousSibling : this;
                pNode.$ext.style.cursor = t && _self.$resizable
                    ? (ppc.isWebkit ? "ew-resize" : "w-resize")
                    : "default";
                
                ppc.setStyleClass(this, "", ["hover", "down"]);
            }
            else {
                pNode.$ext.style.cursor = "default";
                ppc.setStyleClass(this, "hover", ["down"]);
            }
        };

        if (!this.options && pNode.options)
            this.$propHandlers["options"].call(this, 
                this.options = pNode.options);

        if (this.visible === false)
            this.$propHandlers["visible"].call(this, false, null, null, true);
        
        return this;
    }
    
    this.addEventListener("DOMNodeInsertedIntoDocument", function(e){
        this.$draw();
    });
    
    this.addEventListener("DOMNodeRemovedFromDocument", function(e){
        if (!this.$int)
            return;
        
        this.$int.onmouseover   =
        this.$int.onmouseup     =
        this.$int.onmousedown   =
        this.$int.onmousemove   =
        this.$int.onmouseout    = null;
    });
    
}).call(ppc.BindingColumnRule.prototype = new ppc.BindingRule());

ppc.aml.setElement("column", ppc.BindingColumnRule);
// #endif

